require "import"
import "android.content.Context"
import "android.content.Intent"
import "java.util.Locale"

import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"
import "java.util.Locale"
import "android.content.Intent"

local substitutionTable = {
    ["chino"] = "Jieshuo"
}

function capitalizeFirstWord(text)
    local firstChar = text:sub(1, 1)
    local restOfString = text:sub(2)
    return firstChar:upper() .. restOfString
end

function substituteWords(text)
    for word, substitution in pairs(substitutionTable) do
        text = text:gsub("%f[%a]" .. word .. "%f[%A]", substitution)
    end
    return text
end

function capitalizeAfterPeriod(text)
    return text:gsub("(%.[%s]*%a)", function(match)
        local char = match:sub(-1)
        local capitalizedChar = char:upper()
        return match:sub(1, -2) .. capitalizedChar
    end)
end

function lowercaseAfterComma(text)
    return text:gsub("(%,[%s]*%a)", function(match)
        local char = match:sub(-1)
        local lowercaseChar = char:lower()
        return match:sub(1, -2) .. lowercaseChar
    end)
end

function addPunctuation(text)
    local lastChar = text:sub(-1)
    local punctuation = { "", "!", "?" }
    local hasPunctuation = false

    for _, p in ipairs(punctuation) do
        if lastChar == p then
            hasPunctuation = true
            break
        end
    end

    if not hasPunctuation then
        text = text .. ""
    end

    return text
end

function hasInternetConnection()
    local connectivityManager = this.getSystemService(Context.CONNECTIVITY_SERVICE)
    local networkInfo = connectivityManager.getActiveNetworkInfo()
    return networkInfo ~= nil and networkInfo.isConnected()
end

function kill()
    speechRecognizer.destroy()
    speechRecognizer = nil
    return true
end

function startListening()
    if not hasInternetConnection() then
        service.asyncSpeak("No internet connection. Voice recognition is not available.")
        return false
    end
    
    local verbalizationDetected = false
    
    speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)
    speechListener = RecognitionListener{
        onResults = function(results)
            data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if data ~= nil and data.size() > 0 then
                verbalizationDetected = true
                local recognizedText = data.get(0)
                recognizedText = capitalizeFirstWord(recognizedText)
                recognizedText = substituteWords(recognizedText)
                recognizedText = capitalizeAfterPeriod(recognizedText)
                recognizedText = lowercaseAfterComma(recognizedText)
                recognizedText = addPunctuation(recognizedText)

                -- Punctuation markers
                recognizedText = string.gsub(recognizedText, "(%s?)comma(%s?)([%p]*)$", "%2,%3")
                recognizedText = string.gsub(recognizedText, "(%s?)period(%s?)([%p]*)$", "%2.%3")
                recognizedText = string.gsub(recognizedText, "(%s?)question mark(%s?)([%p]*)$", "%2?")
                recognizedText = string.gsub(recognizedText, "(%s?)colon(%s?)([%p]*)$", "%2:")
                recognizedText = string.gsub(recognizedText, "(%s?)quotation marks(%s?)([%p]*)$", "%2\"")

                -- Emoji markers
                recognizedText = string.gsub(recognizedText, "(%s?)winking face(%s?)", "%1\u{1F609}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)smiling face with big eyes(%s?)", "%1\u{1F603}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)crying face(%s?)", "%1\u{1F622}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)face screaming in fear(%s?)", "%1\u{1F631}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)angry face(%s?)", "%1\u{1F621}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)smiling face with halo(%s?)", "%1\u{1F60F}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)disappointed face(%s?)", "%1\u{1F61E}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)smiling face with cold sweat(%s?)", "%1\u{1F605}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)face throwing a kiss(%s?)", "%1\u{1F61A}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)face with hand over mouth(%s?)", "%1\u{1F64A}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)relieved face(%s?)", "%1\u{1F60C}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)smiling face(%s?)", "%1\u{1F600}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)face savoring food(%s?)", "%1\u{1F60B}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)smiling face with smiling eyes(%s?)", "%1\u{1F606}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)thumbs up(%s?)", "%1\u{1F44C}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)neutral face(%s?)", "%1\u{1F610}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)confused face(%s?)", "%1\u{1F615}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)clapping hands(%s?)", "%1\u{1F44F}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)rolling on the floor laughing(%s?)", "%1\u{1F923}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)smiling face with smiling eyes(%s?)", "%1\u{1F60A}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)happy face with smiling eyes(%s?)", "%1\u{1F60A}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)fire emoji(%s?)", "%1\u{1F525}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)poop with eyes(%s?)", "%1\u{1F4A9}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)smiling face with hearts(%s?)", "%1\u{1F60D}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)face with tongue out(%s?)", "%1\u{1F61C}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)face sticking out tongue(%s?)", "%1\u{1F61C}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)red heart(%s?)", "%1\u{2764}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)blue heart(%s?)", "%1\u{1F499}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)green heart(%s?)", "%1\u{1F49A}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)black heart(%s?)", "%1\u{1F5A4}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)face with rolling eyes(%s?)", "%1\u{1F644}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)face tears of joy(%s?)", "%1\u{1F602}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)thumbs up(%s?)", "%1\u{1F44D}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)thumbs down(%s?)", "%1\u{1F44E}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)face with open mouth(%s?)", "%1\u{1F62E}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)angry face(%s?)", "%1\u{1F620}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)folded hands(%s?)", "%1\u{1F64F}%2")
                recognizedText = string.gsub(recognizedText, "(%s?)face with sunglasses(%s?)", "%1\u{1F60E}%2")

                service.getEditText()
                local node = service.getEditText()
                service.insertText(node, recognizedText .. " ")
                service.speak(recognizedText)
            end

            if not verbalizationDetected then
                kill()
                return false
            end
        end,
        onError = function()
            kill()
            return false
        end
    }

    recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "BN_IN")
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())
    speechRecognizer.startListening(recognizerIntent)
    speechRecognizer.setRecognitionListener(speechListener)

    return true
end

startListening()
return true